using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Microsoft.TeamFoundation.Client;
using Microsoft.TeamFoundation.Server;
using Microsoft.TeamFoundation;
using EventSubscriber.Properties;


namespace EventSubscriber
{
    public partial class Subscriber : Form
    {
        TeamFoundationServer server = null;

        public Subscriber()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            txtDomain.Text = System.Environment.UserDomainName;
            txtUserName.Text = System.Environment.UserName;
            txtUnsubUserName.Text = txtUserName.Text;
            txtUnsubDomain.Text = txtDomain.Text;
            txtServerName.Text = Settings.Default.DefaultServer;
            cboDeliverySchedule.SelectedIndex = 0;
            cboDeliveryType.SelectedIndex = 0;
            cboEvent.Items.AddRange(Resources.Events.Split('\n'));
            cboEvent.SelectedIndex = 0;
        }

        private void btnConnect_Click(object sender, EventArgs e)
        {
            try
            {
                this.Cursor = Cursors.WaitCursor;

                server = TeamFoundationServerFactory.GetServer(txtServerName.Text);
                server.Authenticate();
                ICommonStructureService structureService = (ICommonStructureService)server.GetService(typeof(ICommonStructureService));
                ProjectInfo[] projects = structureService.ListAllProjects();
                cboProject.Items.Clear();
                cboProject.Items.Add("(All Projects)");
                foreach (ProjectInfo project in projects)
                {
                    cboProject.Items.Add(project.Name);
                }

                cboProject.SelectedIndex = 0;
                groupBox1.Enabled = true;

                // Select SOAP option as default
                cboDeliveryType.SelectedItem = "E-Mail (HTML)";
                cboEvent.SelectedItem = "CheckinEvent ";
            }
            catch(TeamFoundationInvalidServerNameException isne)
            {
                MessageBox.Show(isne.Message, "Invalid Server Name", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            finally
            {
                this.Cursor = Cursors.Default;
            }
        }

        #region Subscribe
        private void btnSubscribe_Click(object sender, EventArgs e)
        {
            if (server == null) btnConnect_Click(sender, e);
            // check if all fields are valid
            if ((txtUserName.Text.Length > 0) && (txtDomain.Text.Length > 0) && (cboProject.Text.ToString().Length > 0))
            {
                IEventService eventEndpoint = (IEventService)server.GetService(typeof(IEventService));
                DeliveryPreference delPrev = new DeliveryPreference();

                switch (cboDeliveryType.Text)
                {
                    case "E-Mail (HTML)":
                        delPrev.Type = DeliveryType.EmailHtml;
                        break;

                    case "E-Mail (Plain)":
                        delPrev.Type = DeliveryType.EmailPlaintext;
                        break;

                    case "Web Service (SOAP)":
                        delPrev.Type = DeliveryType.Soap;
                        break;

                    default:
                        delPrev.Type = DeliveryType.EmailHtml;
                        break;
                }

                
                delPrev.Schedule = (DeliverySchedule)Enum.Parse(typeof(DeliverySchedule), cboDeliverySchedule.Text);
                delPrev.Address = txtDeliveryAddress.Text;
               
                
                string userId = txtDomain.Text + @"\" + txtUserName.Text;
                string eventType = cboEvent.Text.ToString();
                switch (eventType)
                {
                    case "CheckinPolicyOverride":
                        eventType = "CheckinEvent";
                        break;

                    default:
                        break;
                }
                
                try
                {
                    // The userID will be ignored and only the current program credentials will be used.
                    // http://blogs.infosupport.com/marcelv/archive/2006/04/14/5157.aspx
                    eventEndpoint.SubscribeEvent(userId, eventType, txtFilters.Text, delPrev, cboProject.Text + " : " + cboEvent.Text + " : " + delPrev.Type.ToString());
                    btnSearch_Click(sender, e);
                }
                catch (SystemException ex)
                {
                    MessageBox.Show(ex.Message, "Subscribe error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
            else 
            {
                MessageBox.Show("Please Fill in all required fields (project,username and domain)");
            }
        }
        private void cboProject_SelectedIndexChanged(object sender, EventArgs e)
        {
            cboEvent_SelectedIndexChanged(sender, e);
        }
        private void cboDeliveryType_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (cboDeliveryType.Text.StartsWith("E-Mail"))
                txtDeliveryAddress.Text = System.Environment.UserName + Settings.Default.emailextension;
            else
                txtDeliveryAddress.Text = Settings.Default.EventSubscriber_TFSEvents_Event;
            
        }
        private void cboEvent_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (cboProject.SelectedIndex == 0)
                txtFilters.Text = String.Empty;
            else
            {
                switch (cboEvent.Text)
                {
                    case "CheckinPolicyOverride":
                        txtFilters.Text = "TeamProject = '" + cboProject.Text + "' AND PolicyOverrideComment <> '' ";
                        break;

                    default:
                        txtFilters.Text = "TeamProject = '" + cboProject.Text + "'";
                        break;
                }
            }
        }
        private void btnFilterList_Click(object sender, EventArgs e)
        {
            MessageBox.Show(Properties.Resources.DBFields);
        }
        #endregion

        #region Unsubscribe
        private void btnSearch_Click(object sender, EventArgs e)
        {
            if (server == null) btnConnect_Click(sender, e);

            try
            {
                this.Cursor = Cursors.WaitCursor;
                
                IEventService eventEndpoint = (IEventService)server.GetService(typeof(IEventService));

                // The userID will be ignored and only the current program credentials will be used.
                // http://blogs.infosupport.com/marcelv/archive/2006/04/14/5157.aspx
                Subscription[] subs = eventEndpoint.EventSubscriptions(txtUnsubDomain.Text + "\\" + txtUnsubUserName.Text);

                lstSubscriptions.Items.Clear();
                foreach (Subscription s in subs)
                {
                    lstSubscriptions.Items.Add(s.ID + " : " + s.Tag);
                }
            }
            catch (SystemException ex)
            {
                MessageBox.Show(ex.Message, "Subscribe error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            finally
            {
                this.Cursor = Cursors.Default;
            }
        }
        private void btnRemoveSub_Click(object sender, EventArgs e)
        {
            if (server == null) btnConnect_Click(sender, e);
            IEventService eventEndpoint = (IEventService)server.GetService(typeof(IEventService));

            foreach (string s in lstSubscriptions.SelectedItems)
            {
                int id = Convert.ToInt32(s.Split(':')[0]);
                eventEndpoint.UnsubscribeEvent(id);
            }

            btnSearch_Click(sender, e);
        }
        #endregion


    }
}