using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.TeamFoundation.Build.Proxy;
using TFSWrappers;
using UIComponents;
using System.Windows.Forms;
using System.Drawing;
using System.Diagnostics;

namespace TFSBuildManager
{
    public class BuildReportTabControlItem : BaseTabControlItem
    {
        public BuildReportTabControlItem(BuildStore buildStore, BuildData buildData)
        {
            this.TabType = TabControlType.BuildReport;
            buildReport = new TFSBuildReport(buildStore, buildData.BuildUri);
            this.Title = buildReport.BuildNumber;

            XPPanelGroup xppgReport = new XPPanelGroup();
            xppgReport.PanelGradient.Start = Color.White;
            xppgReport.PanelGradient.End = Color.White;
            xppgReport.Parent = this;
            xppgReport.Dock = DockStyle.Fill;
            xppgReport.Font = this.Font = new Font("Tahoma", 8.25F);

            ImageSet imageSet = new ImageSet();
            imageSet.Images.Add(Properties.Resources.accept);
            imageSet.Images.Add(Properties.Resources.exclamation);
            imageSet.Images.Add(Properties.Resources.warning);

            ImageList imageList = new ImageList();
            imageList.ColorDepth = ColorDepth.Depth32Bit;
            imageList.ImageSize = new Size(16, 16);
            imageList.TransparentColor = Color.Transparent;
            imageList.Images.Add(Properties.Resources.accept);
            imageList.Images.Add(Properties.Resources.exclamation);
            imageList.Images.Add(Properties.Resources.control_play_blue);
            imageList.Images.Add(Properties.Resources.warning);

            #region Summary
            XPPanel xppSummary = CreateXPPanel(xppgReport, "Summary", new Size(797, 320), XPPanelState.Expanded);
            xppSummary.ImageItems.ImageSet = imageSet;
            if (buildReport.FinishedSuccessfully)
            {
                xppSummary.ImageItems.Normal = 0;
                xppSummary.ImageItems.Highlight = 0;
            }
            else
            {
                xppSummary.ImageItems.Normal = 1;
                xppSummary.ImageItems.Highlight = 1;
            }
            CreateLabel(xppSummary, new Point(16, 48), "Build Number:", true);

            lblBuildNumber = new LinkLabel();
            lblBuildNumber.AutoSize = true;
            lblBuildNumber.Parent = xppSummary;
            lblBuildNumber.Location = new Point(144, 48);
            lblBuildNumber.Text = buildReport.BuildNumber;
            lblBuildNumber.Tag = buildData.DropLocation;
            lblBuildNumber.LinkClicked += new LinkLabelLinkClickedEventHandler(lblBuildNumber_LinkClicked);
            
            CreateLabel(xppSummary, new Point(16, 72), "Requested By:", true);
            CreateLabel(xppSummary, new Point(144, 72), buildReport.RequestedBy);
            CreateLabel(xppSummary, new Point(16, 96), "Team Project:", true);
            CreateLabel(xppSummary, new Point(144, 96), buildReport.TeamProject);
            CreateLabel(xppSummary, new Point(16, 120), "Build Type:", true);
            CreateLabel(xppSummary, new Point(144, 120), buildReport.BuildType);
            CreateLabel(xppSummary, new Point(16, 144), "Build Machine:", true);
            CreateLabel(xppSummary, new Point(144, 144), buildReport.BuildMachine);
            CreateLabel(xppSummary, new Point(16, 168), "Started On:", true);
            CreateLabel(xppSummary, new Point(144, 168), buildReport.StartedOn.ToString());
            CreateLabel(xppSummary, new Point(16, 192), "Completed On:", true);
            CreateLabel(xppSummary, new Point(144, 192), buildReport.CompletedOn.ToString());
            CreateLabel(xppSummary, new Point(16, 216), "Last Changed By:", true);
            CreateLabel(xppSummary, new Point(144, 216), buildReport.LastChangedBy);
            CreateLabel(xppSummary, new Point(16, 240), "Last Changed On:", true);
            CreateLabel(xppSummary, new Point(144, 240), buildReport.LastChangedOn.ToString());
            CreateLabel(xppSummary, new Point(16, 264), "Quality:", true);
            CreateLabel(xppSummary, new Point(144, 264), buildReport.Quality);

            CreateLabel(xppSummary, new Point(16, 288), "Log File:", true);

            lblLog = new LinkLabel();
            lblLog.Parent = xppSummary;
            lblLog.AutoSize = true;
            lblLog.Location = new Point(144, 288);
            lblLog.Text = buildReport.LogFile;
            lblLog.LinkClicked += new LinkLabelLinkClickedEventHandler(lblLog_LinkClicked);
            #endregion

            #region Build Steps
            XPPanel xppBuildSteps = CreateXPPanel(xppgReport, "Build Steps (" + buildReport.BuildSteps.Length.ToString() + ")", new Size(797, 320), XPPanelState.Expanded);
            xppBuildSteps.PanelState = XPPanelState.Collapsed;
            xppBuildSteps.ImageItems.ImageSet = imageSet;
            if (buildReport.FinishedSuccessfully)
            {
                xppBuildSteps.ImageItems.Normal = 0;
                xppBuildSteps.ImageItems.Highlight = 0;
            }
            else
            {
                xppBuildSteps.ImageItems.Normal = 1;
                xppBuildSteps.ImageItems.Highlight = 1;
            }

            buildStepsListView = new ListView();
            buildStepsListView.FullRowSelect = true;
            buildStepsListView.HideSelection = false;
            buildStepsListView.SmallImageList = imageList;
            buildStepsListView.Parent = xppBuildSteps;
            buildStepsListView.Dock = DockStyle.Fill;
            buildStepsListView.Columns.Add("Build Step");
            buildStepsListView.Columns.Add("Completed On", 120);
            buildStepsListView.View = View.Details;
            buildStepsListView.Resize += new EventHandler(buildStepsListView_Resize);
            buildStepsListView_Resize(null, null);
            LoadBuildSteps();
            xppBuildSteps.Height = buildStepsListView.Items.Count * 20;
            #endregion

            #region Compilation Details

            foreach (PlatformFlavorData plaformFlavor in buildReport.PlatformFlavors)
            {
                CompilationSummaryData compilationSummary = buildStore.GetCompilationSummaryForBuild(
                    buildData.BuildUri, plaformFlavor.PlatformName, plaformFlavor.FlavorName);
                long totalErrors = compilationSummary.TotalCompilerErrors + compilationSummary.TotalCodeAnalysisErrors;
                long totalWarnings = compilationSummary.TotalCompilerWarnings + compilationSummary.TotalCodeAnalysisWarnings;
                XPPanel xppPlatformFlavor = CreateXPPanel(xppgReport, plaformFlavor.PlatformName + "|" + 
                    plaformFlavor.FlavorName, new Size(300, 220), XPPanelState.Expanded);
                xppPlatformFlavor.PanelState = XPPanelState.Collapsed;
                xppPlatformFlavor.ImageItems.ImageSet = imageSet;
                if (!buildReport.FinishedSuccessfully)
                {
                    xppPlatformFlavor.ImageItems.Normal = 1;
                    xppPlatformFlavor.ImageItems.Highlight = 1;
                }
                else if (totalWarnings > 0)
                {
                    xppPlatformFlavor.ImageItems.Normal = 2;
                    xppPlatformFlavor.ImageItems.Highlight = 2;
                }
                else
                {
                    xppPlatformFlavor.ImageItems.Normal = 0;
                    xppPlatformFlavor.ImageItems.Highlight = 0;
                }

                LinkLabel lblTotalErrorsAndWarnings = new LinkLabel();
                lblTotalErrorsAndWarnings.AutoSize = true;
                lblTotalErrorsAndWarnings.Parent = xppPlatformFlavor;
                lblTotalErrorsAndWarnings.Location = new Point(16, 48);
                lblTotalErrorsAndWarnings.Text = totalErrors.ToString() + " error(s), " + totalWarnings.ToString() + " warning(s)";
                lblTotalErrorsAndWarnings.Tag = compilationSummary.LogFileName;
                lblTotalErrorsAndWarnings.LinkClicked += new LinkLabelLinkClickedEventHandler(lblTotalErrorsAndWarnings_LinkClicked);

                CreateLabel(xppPlatformFlavor, new Point(16, 72), "Compiler Errors:", true);
                CreateLabel(xppPlatformFlavor, new Point(164, 72), compilationSummary.TotalCompilerErrors.ToString());
                CreateLabel(xppPlatformFlavor, new Point(16, 96), "Compiler Warnings:", true);
                CreateLabel(xppPlatformFlavor, new Point(164, 96), compilationSummary.TotalCompilerWarnings.ToString());
                CreateLabel(xppPlatformFlavor, new Point(16, 120), "Code Analysis Errors:", true);
                CreateLabel(xppPlatformFlavor, new Point(164, 120), compilationSummary.TotalCodeAnalysisErrors.ToString());
                CreateLabel(xppPlatformFlavor, new Point(16, 144), "Code Analysis Warnings:", true);
                CreateLabel(xppPlatformFlavor, new Point(164, 144), compilationSummary.TotalCodeAnalysisWarnings.ToString());
                CreateLabel(xppPlatformFlavor, new Point(16, 168), "Total Errors:", true);
                CreateLabel(xppPlatformFlavor, new Point(164, 168), totalErrors.ToString());
                CreateLabel(xppPlatformFlavor, new Point(16, 192), "Total Warnings:", true);
                CreateLabel(xppPlatformFlavor, new Point(164, 192), totalWarnings.ToString());
            }

            #endregion

            #region Changesets
            XPPanel xppChangesets = CreateXPPanel(xppgReport, "Associated Changesets (" + buildReport.Changesets.Length.ToString() + ")", new Size(797, 320), XPPanelState.Expanded);
            xppChangesets.PanelState = XPPanelState.Collapsed;
            changesetsListView = new ListView();
            changesetsListView.FullRowSelect = true;
            changesetsListView.HideSelection = false;
            changesetsListView.SmallImageList = imageList;
            changesetsListView.Parent = xppChangesets;
            changesetsListView.Dock = DockStyle.Fill;
            changesetsListView.Columns.Add("Id", 80);
            changesetsListView.Columns.Add("Checkedin By", 120);
            changesetsListView.Columns.Add("Comment");
            changesetsListView.View = View.Details;
            changesetsListView.Resize += new EventHandler(changesetsListView_Resize);
            changesetsListView_Resize(null, null);
            LoadChangesets();
            #endregion

            #region Work Items
            XPPanel xppWorkItems = CreateXPPanel(xppgReport, "Associated Work Items (" + buildReport.WorkItems.Length.ToString() + ")", new Size(797, 320), XPPanelState.Expanded);
            xppWorkItems.PanelState = XPPanelState.Collapsed;
            workItemsListView = new ListView();
            workItemsListView.FullRowSelect = true;
            workItemsListView.HideSelection = false;
            workItemsListView.SmallImageList = imageList;
            workItemsListView.Parent = xppWorkItems;
            workItemsListView.Dock = DockStyle.Fill;
            workItemsListView.Columns.Add("Id", 80);
            workItemsListView.Columns.Add("Title");
            workItemsListView.Columns.Add("Assigned To", 120);
            workItemsListView.Columns.Add("Status", 120);
            workItemsListView.View = View.Details;
            workItemsListView.Resize += new EventHandler(workItemsListView_Resize);
            workItemsListView_Resize(null, null);
            workItemsListView.DoubleClick += new EventHandler(workItemsListView_DoubleClick);
            LoadWorkItems();
            #endregion
        }

        void workItemsListView_DoubleClick(object sender, EventArgs e)
        {
            if (workItemsListView.SelectedItems.Count == 1)
            {
                WorkItemData workItemData = (WorkItemData)workItemsListView.SelectedItems[0].Tag;
                Process.Start(workItemData.WorkItemUri);
            }
        }

        void lblTotalErrorsAndWarnings_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
               
        }

        private XPPanel CreateXPPanel(Control parent,  string caption, Size size, XPPanelState panelState)
        {
            XPPanel xpPanel = new XPPanel();
            xpPanel.PanelState = panelState;
            xpPanel.CaptionGradient.Start = Color.Gray;
            xpPanel.CaptionGradient.End = Color.Gray;
            xpPanel.PanelGradient.Start = Color.White;
            xpPanel.PanelGradient.End = Color.White;
            xpPanel.TextColors.Foreground = Color.White;
            xpPanel.TextColors.Background = Color.White;
            xpPanel.TextHighlightColors.Foreground = Color.LightYellow;
            xpPanel.TextHighlightColors.Background = Color.LightYellow;
            xpPanel.Font = this.Font = new Font("Tahoma", 8.25F);
            xpPanel.Parent = parent;
            xpPanel.Anchor = ((AnchorStyles)(((AnchorStyles.Top | AnchorStyles.Left)
                        | AnchorStyles.Right)));
            xpPanel.Caption = caption;
            xpPanel.Size = size;
            xpPanel.AnimationRate = 0;
            xpPanel.XPPanelStyle = XPPanelStyle.WindowsXP;
            return xpPanel;
        }

        void changesetsListView_Resize(object sender, EventArgs e)
        {
            changesetsListView.Columns[2].Width = changesetsListView.Width - changesetsListView.Columns[0].Width -
                changesetsListView.Columns[1].Width - 20;            
        }

        void workItemsListView_Resize(object sender, EventArgs e)
        {
            workItemsListView.Columns[1].Width = workItemsListView.Width - workItemsListView.Columns[0].Width -
                workItemsListView.Columns[2].Width - workItemsListView.Columns[3].Width - 20;
        }

        void buildStepsListView_Resize(object sender, EventArgs e)
        {
            buildStepsListView.Columns[0].Width = buildStepsListView.Width - buildStepsListView.Columns[1].Width - 20;
        }

        private TFSBuildReport buildReport;
        private LinkLabel lblLog;
        private LinkLabel lblBuildNumber;
        private ListView buildStepsListView;
        private ListView changesetsListView;
        private ListView workItemsListView;

        private void LoadChangesets()
        {
            changesetsListView.Items.Clear();
            foreach (ChangeSetData changeset in buildReport.Changesets)
            {
                ListViewItem listViewItem = new ListViewItem(changeset.ChangeSetId.ToString());
                listViewItem.SubItems.Add(changeset.CheckedInBy);
                listViewItem.SubItems.Add(changeset.Comment);
                listViewItem.Tag = changeset;
                changesetsListView.Items.Add(listViewItem);
            }
        }

        private void LoadBuildSteps()
        {
            buildStepsListView.Items.Clear();
            foreach (BuildStepData buildStep in buildReport.BuildSteps)
            {
                ListViewItem listViewItem = new ListViewItem(buildStep.BuildStepMessage);
                listViewItem.SubItems.Add(buildStep.FinishTime.ToString());
                if (buildStep.Status == BuildStepStatus.Failed)
                {
                    listViewItem.ImageIndex = 1;
                }
                else if (buildStep.Status == BuildStepStatus.InProgress)
                {
                    listViewItem.ImageIndex = 2;
                }
                else if (buildStep.Status == BuildStepStatus.Succeeded)
                {
                    listViewItem.ImageIndex = 0;
                }
                listViewItem.Tag = buildStep;
                buildStepsListView.Items.Add(listViewItem);
            }
        }

        private void LoadWorkItems()
        {
            workItemsListView.Items.Clear();
            foreach (WorkItemData workItem in buildReport.WorkItems)
            {
                ListViewItem listViewItem = new ListViewItem(workItem.WorkItemId.ToString());
                listViewItem.SubItems.Add(workItem.Title);
                listViewItem.SubItems.Add(workItem.AssignedTo);
                listViewItem.SubItems.Add(workItem.Status);
                listViewItem.Tag = workItem;
                workItemsListView.Items.Add(listViewItem);
            }
        }

        private Label CreateLabel(Control parent, Point location, string text)
        {
            return CreateLabel(parent, location, text, false);
        }

        private Label CreateLabel(Control parent, Point location, string text, bool bold)
        {
            Label label = new Label();
            label.AutoSize = true;
            label.Parent = parent;
            label.Location = location;
            label.Text = text;
            if (bold)
            {
                label.Font = new Font("Tahoma", 8.25F, FontStyle.Bold);
            }
            else
            {
                label.Font = new Font("Tahoma", 8.25F);
            }
            return label;
        }

        public event LogFileLinkLabelClickedHandler LogFileLinkLabelClicked;

        public void lblLog_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            if (LogFileLinkLabelClicked != null)
            {
                LogFileLinkLabelClicked(new LogFileLinkLabelClickedEventArgs(lblLog.Text));
            }
        }

        void lblBuildNumber_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            Process.Start("explorer", "/e," + (string)lblBuildNumber.Tag);
        }
    }

    public class LogFileLinkLabelClickedEventArgs : EventArgs
    {
        public LogFileLinkLabelClickedEventArgs(string logFile)
        {
            this.logFile = logFile;
        }

        private string logFile;

        public string LogFile
        {
            get { return logFile; }
        }
    }

    public delegate void LogFileLinkLabelClickedHandler(LogFileLinkLabelClickedEventArgs e);
}
