using System;
using System.Collections.Generic;
using System.Text;
using TFSWrappers;
using System.Windows.Forms;
using System.Drawing;
using System.IO;
using System.Diagnostics;
using Microsoft.TeamFoundation.Build.Proxy;
using Microsoft.TeamFoundation.Build.Client;
using Microsoft.TeamFoundation.VersionControl.Client;
using Microsoft.TeamFoundation.WorkItemTracking.Client;

namespace TFSBuildManager
{
    public class BuildTypeTabControlItem : BaseTabControlItem
    {
        public BuildTypeTabControlItem(SourceControlWrapper sourceControlWrapper,
            WorkspaceWrapper workspaceWrapper, BuildStoreWrapper buildStoreWrapper,
            string projectName, string buildTypeName, TreeNode tabTreeNode,
            string buildTypeFileViewer,
            string logFileViewer)
        {
            this.TabType = TabControlType.BuildType;
            this.Title = buildTypeName;
            this.Name = buildTypeName.Replace(" ", "_");
            this.tabTreeNode = tabTreeNode;
            this.sourceControlWrapper = sourceControlWrapper;
            this.workspaceWrapper = workspaceWrapper;
            this.buildStoreWrapper = buildStoreWrapper;
            this.projectName = projectName;
            this.buildTypeName = buildTypeName;
            this.buildTypeFileViewer = buildTypeFileViewer;
            this.logFileViewer = logFileViewer;


            #region Panel
            Panel panel = new Panel();
            panel.Parent = this;
            panel.BackColor = SystemColors.Info;
            panel.Height = 23;
            panel.Dock = DockStyle.Top;
            panel.BorderStyle = BorderStyle.FixedSingle;
            PictureBox pictureBox = new PictureBox();
            pictureBox.SizeMode = PictureBoxSizeMode.AutoSize;
            pictureBox.Image = Properties.Resources.information;
            pictureBox.Parent = panel;
            pictureBox.Location = new Point(4, 2);
            //Information panel label
            tabLastRefreshedLabel = new Label();
            tabLastRefreshedLabel.AutoSize = true;
            tabLastRefreshedLabel.Text = "Last refreshed on...";
            tabLastRefreshedLabel.Parent = panel;
            tabLastRefreshedLabel.Location = new Point(25, 2);
            #endregion

            #region ToolStrip
            ToolStrip toolStrip = new ToolStrip();

            ToolStripButton refreshToolStripButton = new ToolStripButton("", Properties.Resources.table_refresh, refresh_Click);
            refreshToolStripButton.ToolTipText = "Refresh";
            toolStrip.Items.Add(refreshToolStripButton);

            toolStrip.Items.Add(new ToolStripSeparator());

            executeToolStripButton = new ToolStripButton("", Properties.Resources.control_play_blue, execute_Click);
            executeToolStripButton.ToolTipText = "Execute";
            toolStrip.Items.Add(executeToolStripButton);

            customExecuteToolStripButton = new ToolStripButton("", Properties.Resources.application_lightning, customExecute_Click);
            customExecuteToolStripButton.ToolTipText = "Custom Execute";
            toolStrip.Items.Add(customExecuteToolStripButton);

            executeWithoutGetToolStripButton = new ToolStripButton("", Properties.Resources.cog_delete, executeWithoutGet_Click);
            executeWithoutGetToolStripButton.ToolTipText = "Execute Without Get";
            toolStrip.Items.Add(executeWithoutGetToolStripButton);

            stopToolStripButton = new ToolStripButton("", Properties.Resources.stop, stop_Click);
            stopToolStripButton.Enabled = false;
            stopToolStripButton.ToolTipText = "Stop";
            toolStrip.Items.Add(stopToolStripButton);

            toolStrip.Items.Add(new ToolStripSeparator());

            editToolStripButton = new ToolStripButton("", Properties.Resources.application_edit, edit_Click);
            editToolStripButton.ToolTipText = "Edit Build Type";
            toolStrip.Items.Add(editToolStripButton);

            viewFileToolStripButton = new ToolStripButton("", Properties.Resources.page_white_magnify, view_Click);
            viewFileToolStripButton.ToolTipText = "View Build Type File";
            toolStrip.Items.Add(viewFileToolStripButton);

            toolStrip.Items.Add(new ToolStripSeparator());

            changeQualityToolStripButton = new ToolStripButton("", Properties.Resources.accept, changeQuality_Click);
            changeQualityToolStripButton.Enabled = false;
            changeQualityToolStripButton.ToolTipText = "Change Quality";
            toolStrip.Items.Add(changeQualityToolStripButton);

            deleteToolStripButton = new ToolStripButton("", Properties.Resources.delete, delete_Click);
            deleteToolStripButton.Enabled = false;
            deleteToolStripButton.ToolTipText = "Delete";
            toolStrip.Items.Add(deleteToolStripButton);

            exploreDropLocationToolStripButton = new ToolStripButton("", Properties.Resources.folder_explore, exploreDropLocation_Click);
            exploreDropLocationToolStripButton.Enabled = false;
            exploreDropLocationToolStripButton.ToolTipText = "Explore Drop Location";
            toolStrip.Items.Add(exploreDropLocationToolStripButton);

            viewLogFileToolStripButton = new ToolStripButton("", Properties.Resources.page_error, viewLogFile_Click);
            viewLogFileToolStripButton.Enabled = false;
            viewLogFileToolStripButton.ToolTipText = "View Log File";
            toolStrip.Items.Add(viewLogFileToolStripButton);

            viewDetailsToolStripButton = new ToolStripButton("", Properties.Resources.information, viewDetails_Click);
            viewDetailsToolStripButton.Enabled = false;
            viewDetailsToolStripButton.ToolTipText = "View Details";
            toolStrip.Items.Add(viewDetailsToolStripButton);

            toolStrip.Parent = this;
            #endregion

            #region ContextMenuStrip
            ContextMenuStrip contextMenuStrip = new ContextMenuStrip();
            stopToolStripMenuItem = new ToolStripMenuItem("Stop", Properties.Resources.stop);
            stopToolStripMenuItem.Enabled = false;
            stopToolStripMenuItem.Click += stop_Click;
            contextMenuStrip.Items.Add(stopToolStripMenuItem);
            deleteToolStripMenuItem = new ToolStripMenuItem("Delete", Properties.Resources.delete);
            deleteToolStripMenuItem.Enabled = false;
            deleteToolStripMenuItem.Click += delete_Click;
            contextMenuStrip.Items.Add(deleteToolStripMenuItem);
            changeQualityToolStripMenuItem = new ToolStripMenuItem("Change Quality", Properties.Resources.accept);
            changeQualityToolStripMenuItem.Enabled = false;
            changeQualityToolStripMenuItem.Click += changeQuality_Click;
            contextMenuStrip.Items.Add(changeQualityToolStripMenuItem);
            exploreDropLocationToolStripMenuItem = new ToolStripMenuItem("Explore Drop Location", Properties.Resources.folder_explore);
            exploreDropLocationToolStripMenuItem.Enabled = false;
            exploreDropLocationToolStripMenuItem.Click += exploreDropLocation_Click;
            contextMenuStrip.Items.Add(exploreDropLocationToolStripMenuItem);
            viewLogFileToolStripMenuItem = new ToolStripMenuItem("View Log File", Properties.Resources.page_error);
            viewLogFileToolStripMenuItem.Enabled = false;
            viewLogFileToolStripMenuItem.Click += viewLogFile_Click;
            contextMenuStrip.Items.Add(viewLogFileToolStripMenuItem);
            viewDetailsToolStripMenuItem = new ToolStripMenuItem("View Details", Properties.Resources.information);
            viewDetailsToolStripMenuItem.Enabled = false;
            viewDetailsToolStripMenuItem.Click += viewDetails_Click;
            contextMenuStrip.Items.Add(viewDetailsToolStripMenuItem);
            #endregion

            #region ImageList
            ImageList imageList = new ImageList();
            imageList.ColorDepth = ColorDepth.Depth32Bit;
            imageList.TransparentColor = Color.Transparent;
            imageList.Images.Add(Properties.Resources.control_play_blue);
            imageList.Images.Add(Properties.Resources.accept);
            imageList.Images.Add(Properties.Resources.exclamation);
            imageList.Images.Add(Properties.Resources.stop);
            #endregion

            #region ListView
            this.tabListView = new ListView();
            tabListView.Parent = this;
            tabListView.Dock = DockStyle.Fill;
            tabListView.View = View.Details;
            tabListView.FullRowSelect = true;
            tabListView.HideSelection = false;
            tabListView.MultiSelect = true;
            tabListView.SmallImageList = imageList;
            tabListView.ContextMenuStrip = contextMenuStrip;
            tabListView.Columns.Add(new ColumnHeader());
            tabListView.Columns.Add(new ColumnHeader());
            tabListView.Columns.Add(new ColumnHeader());
            tabListView.Columns.Add(new ColumnHeader());
            tabListView.Columns[0].Text = "Build Number";
            tabListView.Columns[1].Text = "Build Status";
            tabListView.Columns[2].Text = "Build Quality";
            tabListView.Columns[3].Text = "Completed On";
            tabListView.Columns[0].Width = 200;
            tabListView.Columns[1].Width = 150;
            tabListView.Columns[2].Width = 175;
            tabListView.Columns[3].Width = 250;
            tabListView.Resize += new EventHandler(tabListView_Resize);
            tabListView_Resize(tabListView, new EventArgs());
            tabListView.ColumnClick += new ColumnClickEventHandler(tabListView_ColumnClick);
            tabListView.SelectedIndexChanged += new EventHandler(tabListView_SelectedIndexChanged);
            tabListView.DoubleClick += new EventHandler(tabListView_DoubleClick);
            tabListView.BringToFront();
            listViewColumnSorter = new ListViewColumnSorter();
            tabListView.ListViewItemSorter = listViewColumnSorter;
            #endregion

            #region StatusBar
            StatusStrip statusStrip = new StatusStrip();
            statusStrip.Parent = this;
            statusStrip.Items.Add(new ToolStripStatusLabel("Build Machine:"));
            tabBuildMachineLabel = new ToolStripStatusLabel("");
            tabBuildMachineLabel.ForeColor = Color.Blue;
            statusStrip.Items.Add(tabBuildMachineLabel);
            statusStrip.Items.Add(new ToolStripStatusLabel("Build Directory:"));
            tabBuildDirectoryLabel = new ToolStripStatusLabel("");
            tabBuildDirectoryLabel.ForeColor = Color.Blue;
            statusStrip.Items.Add(tabBuildDirectoryLabel);
            statusStrip.Items.Add(new ToolStripStatusLabel("Drop Location:"));
            tabDropLocationLabel = new ToolStripStatusLabel("");
            tabDropLocationLabel.ForeColor = Color.Blue;
            statusStrip.Items.Add(tabDropLocationLabel);
            statusStrip.Items.Add(new ToolStripStatusLabel("Status:"));
            tabBuildStatusLabel = new ToolStripStatusLabel("");
            tabBuildStatusLabel.ForeColor = Color.Blue;
            statusStrip.Items.Add(tabBuildStatusLabel);
            #endregion

            RefreshView();

            bool notAllBuildTypes = TabBuildType.Name != TFSWrappers.Constants.AllBuildTypes;
            executeToolStripButton.Enabled = notAllBuildTypes;
            customExecuteToolStripButton.Enabled = notAllBuildTypes;
            executeWithoutGetToolStripButton.Enabled = notAllBuildTypes;
            editToolStripButton.Enabled = notAllBuildTypes;
            viewFileToolStripButton.Enabled = notAllBuildTypes;
        }

        #region Private Fields
        private ToolStripButton executeToolStripButton;
        private ToolStripButton customExecuteToolStripButton;
        private ToolStripButton executeWithoutGetToolStripButton;
        private ToolStripButton editToolStripButton;
        private ToolStripButton viewFileToolStripButton;
        private ToolStripButton stopToolStripButton;
        private ToolStripButton deleteToolStripButton;
        private ToolStripButton changeQualityToolStripButton;
        private ToolStripButton exploreDropLocationToolStripButton;
        private ToolStripButton viewLogFileToolStripButton;
        private ToolStripButton viewDetailsToolStripButton;
        private ToolStripMenuItem stopToolStripMenuItem;
        private ToolStripMenuItem deleteToolStripMenuItem;
        private ToolStripMenuItem changeQualityToolStripMenuItem;
        private ToolStripMenuItem exploreDropLocationToolStripMenuItem;
        private ToolStripMenuItem viewLogFileToolStripMenuItem;
        private ToolStripMenuItem viewDetailsToolStripMenuItem;
        private bool initialSortDone;
        private ListViewColumnSorter listViewColumnSorter;
        private SourceControlWrapper sourceControlWrapper;
        private WorkspaceWrapper workspaceWrapper;
        private BuildStoreWrapper buildStoreWrapper;
        private string projectName;
        private string buildTypeName;
        private string buildTypeFileViewer;
        private string logFileViewer;
        #endregion

        #region Properties
        private TreeNode tabTreeNode;

        public TreeNode TabTreeNode
        {
            get { return tabTreeNode; }
        }

        private TFSBuildType tabBuildType;

        public TFSBuildType TabBuildType
        {
            get { return tabBuildType; }
        }

        private ListView tabListView;

        public ListView TabListView
        {
            get { return tabListView; }
        }

        private Label tabLastRefreshedLabel;

        public Label TabLastRefreshedLabel
        {
            get { return tabLastRefreshedLabel; }
        }

        private ToolStripStatusLabel tabBuildMachineLabel;

        public ToolStripStatusLabel TabBuildMachineLabel
        {
            get { return tabBuildMachineLabel; }
        }

        private ToolStripStatusLabel tabBuildStatusLabel;

        public ToolStripStatusLabel TabBuildStatusLabel
        {
            get { return tabBuildStatusLabel; }
        }

        private ToolStripStatusLabel tabBuildDirectoryLabel;

        public ToolStripStatusLabel TabBuildDirectoryLabel
        {
            get { return tabBuildDirectoryLabel; }
        }

        private ToolStripStatusLabel tabDropLocationLabel;

        public ToolStripStatusLabel TabDropLocationLabel
        {
            get { return tabDropLocationLabel; }
        }

        #endregion

        void tabListView_DoubleClick(object sender, EventArgs e)
        {
            if (TabListView.SelectedItems.Count == 1)
            {
                ViewDetails();
            }
        }

        void tabListView_SelectedIndexChanged(object sender, EventArgs e)
        {
            bool oneOfSelectedBuildsIsRunning = false;
            foreach (ListViewItem listViewItem in TabListView.SelectedItems)
            {
                BuildData buildData = ((BuildData)TabListView.SelectedItems[0].Tag);
                if (buildData.BuildStatusId ==  0)
                {
                    oneOfSelectedBuildsIsRunning = true;
                    break;
                }
            }

            int buildStatusId = 0;
            if (TabListView.SelectedItems.Count == 1)
            {
                buildStatusId = ((BuildData)TabListView.SelectedItems[0].Tag).BuildStatusId;
            }

            bool oneBuildSelected = TabListView.SelectedItems.Count == 1;
            bool buildsSelected = TabListView.SelectedItems.Count > 0;
            stopToolStripButton.Enabled = (oneBuildSelected && oneOfSelectedBuildsIsRunning);
            deleteToolStripButton.Enabled = (buildsSelected && !oneOfSelectedBuildsIsRunning);
            changeQualityToolStripButton.Enabled = (buildsSelected && !oneOfSelectedBuildsIsRunning);
            exploreDropLocationToolStripButton.Enabled = oneBuildSelected;
            viewLogFileToolStripButton.Enabled = oneBuildSelected;
            viewDetailsToolStripButton.Enabled = oneBuildSelected;
            stopToolStripMenuItem.Enabled = (oneBuildSelected && oneOfSelectedBuildsIsRunning);
            deleteToolStripMenuItem.Enabled = (buildsSelected && !oneOfSelectedBuildsIsRunning);
            changeQualityToolStripMenuItem.Enabled = (buildsSelected && !oneOfSelectedBuildsIsRunning);
            exploreDropLocationToolStripMenuItem.Enabled = oneBuildSelected;
            viewLogFileToolStripMenuItem.Enabled = oneBuildSelected;
            viewDetailsToolStripMenuItem.Enabled = oneBuildSelected;
        }

        void tabListView_ColumnClick(object sender, ColumnClickEventArgs e)
        {
            if (e.Column == listViewColumnSorter.SortColumn)
            {
                if (listViewColumnSorter.Order == SortOrder.Ascending)
                {
                    listViewColumnSorter.Order = SortOrder.Descending;
                }
                else
                {
                    listViewColumnSorter.Order = SortOrder.Ascending;
                }
            }
            else
            {
                listViewColumnSorter.SortColumn = e.Column;
                listViewColumnSorter.Order = SortOrder.Ascending;
            }
            tabListView.Sort();
        }

        void tabListView_Resize(object sender, EventArgs e)
        {
            ListView listView = (ListView)sender;
            listView.Columns[3].Width = listView.Width - listView.Columns[0].Width - listView.Columns[1].Width - listView.Columns[2].Width - 20;
        }

        private void LoadBuildsList()
        {
            tabListView.Items.Clear();
            if (tabBuildType != null)
            {
                tabListView.BeginUpdate();
                try
                {
                    foreach (BuildData build in tabBuildType.BuildsData)
                    {
                        ListViewItem listViewItem = new ListViewItem();
                        listViewItem.Tag = build;
                        listViewItem.Text = build.BuildNumber;
                        listViewItem.SubItems.Add(build.BuildStatus);
                        listViewItem.SubItems.Add(build.BuildQuality);
                        listViewItem.SubItems.Add(build.FinishTime.ToString());
                        listViewItem.ImageIndex = build.BuildStatusId / 100;
                        tabListView.Items.Add(listViewItem);
                    }
                }
                finally
                {
                    tabListView.EndUpdate();
                }
                if (!initialSortDone)
                {
                    tabListView_ColumnClick(null, new ColumnClickEventArgs(0));
                    tabListView_ColumnClick(null, new ColumnClickEventArgs(0));
                    initialSortDone = true;
                }
                tabListView_SelectedIndexChanged(null, null);
            }
        }

        private void ViewBuildTypeFile()
        {
            try
            {
                if ((buildTypeFileViewer == null) || (buildTypeFileViewer == String.Empty))
                {
                    buildTypeFileViewer = "notepad";
                }
                if (File.Exists(TabBuildType.FileLocalPath))
                {
                    Process.Start(buildTypeFileViewer, TabBuildType.FileLocalPath);
                }
                else
                {
                    MessageBox.Show("Build type file does not exist.", "TFSBuildMnager", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show("Error opening log file. " + ex.Message, "TFSBuildMnager", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void view_Click(object sender, EventArgs e)
        {
            ViewBuildTypeFile();
        }

        private void EditBuildType()
        {
            EditBuildTypeForm frmEditBuildType = new EditBuildTypeForm(sourceControlWrapper.TFS, TabBuildType);
            if (frmEditBuildType.ShowDialog() == DialogResult.OK)
            {
                RefreshView();
            }
        }

        private void edit_Click(object sender, EventArgs e)
        {
            EditBuildType();
        }

        private void ExecuteBuild()
        {
            if (MessageBox.Show("Are you sure that you want to start a build?", "TFSBuildManager",
                MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
            {
                Cursor = Cursors.WaitCursor;
                try
                {
                    TabBuildType.Execute(true);
                }
                catch (Exception ex)
                {
                    MessageBox.Show("Error starting build. " + ex.Message, "TFSBuildMnager", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                finally
                {
                    Cursor = Cursors.Default;
                    RefreshView();
                }
            }
        }

        private void execute_Click(object sender, EventArgs e)
        {
            ExecuteBuild();
        }

        private void StopBuild()
        {
            BuildData buildData = (BuildData)TabListView.SelectedItems[0].Tag;
            if (MessageBox.Show(String.Format("Are you sure that you want to stop build {0} ?",
                buildData.BuildNumber), "Stop Build", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
            {
                string failureMessage = null;
                Cursor = Cursors.WaitCursor;
                try
                {
                    BuildController buildController = (BuildController)sourceControlWrapper.TFS.GetService(typeof(BuildController));
                    buildController.StopBuild(buildData.BuildUri, out failureMessage);
                }
                finally
                {
                    Cursor = Cursors.Default;
                    if (failureMessage != null)
                    {
                        MessageBox.Show(failureMessage, "Stop Build", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                    RefreshView();
                }
            }
        }

        private void stop_Click(object sender, EventArgs e)
        {
            StopBuild();
        }

        public void RefreshView()
        {
            if (tabBuildType != null)
            {
                tabBuildType.Dispose();
            }
            tabBuildType = new TFSBuildType(sourceControlWrapper, workspaceWrapper, buildStoreWrapper, projectName, buildTypeName);
            LoadBuildsList();
            tabLastRefreshedLabel.Text = String.Format("Last refreshed on {0}.", DateTime.Now.ToString());
            tabBuildMachineLabel.Text = tabBuildType.BuildMachine.PropertyValue;
            tabBuildStatusLabel.Text = tabBuildType.GetBuildStatus().ToString();
            tabBuildDirectoryLabel.Text = tabBuildType.BuildDirectoryPath.PropertyValue;
            tabDropLocationLabel.Text = tabBuildType.DropLocation.PropertyValue;
        }

        private void refresh_Click(object sender, EventArgs e)
        {
            RefreshView();
        }

        private void DeleteBuilds()
        {
            List<BuildData> buildsData = new List<BuildData>();
            foreach (ListViewItem listViewItem in TabListView.SelectedItems)
            {
                buildsData.Add((BuildData)listViewItem.Tag);
            }
            DeleteBuildsParametersForm frmDeleteBuildsParameters = new DeleteBuildsParametersForm(sourceControlWrapper.TFS, TabBuildType, buildsData.ToArray());
            if (frmDeleteBuildsParameters.ShowDialog() == DialogResult.OK)
            {
                RefreshView();
            }
        }

        private void delete_Click(object sender, EventArgs e)
        {
            DeleteBuilds();
        }

        private void CustomExecute()
        {
            BuildParametersForm frmBuildParameters = new BuildParametersForm(TabBuildType);
            if (frmBuildParameters.ShowDialog() == DialogResult.OK)
            {
                Cursor = Cursors.WaitCursor;
                try
                {
                    TabBuildType.Execute(frmBuildParameters.BuildMachine,
                        frmBuildParameters.DropLocation, true);
                }
                finally
                {
                    Cursor = Cursors.Default;
                    RefreshView();
                }
            }
        }

        private void customExecute_Click(object sender, EventArgs e)
        {
            CustomExecute();
        }

        private void ChangeQuality()
        {
            ChangeBuildQualityForm frmUpdateBuildQuality = new ChangeBuildQualityForm(
                TabBuildType.ProjectName, buildStoreWrapper);
            if (frmUpdateBuildQuality.ShowDialog() == DialogResult.OK)
            {
                Cursor = Cursors.WaitCursor;
                try
                {
                    foreach (ListViewItem listViewItem in TabListView.SelectedItems)
                    {
                        BuildData buildData = (BuildData)listViewItem.Tag;
                        buildStoreWrapper.UpdateBuildQuality(TabBuildType.ProjectName, buildData.BuildNumber,
                            frmUpdateBuildQuality.SelectedBuildQuality);
                        listViewItem.SubItems[2].Text = frmUpdateBuildQuality.SelectedBuildQuality;
                    }
                }
                finally
                {
                    Cursor = Cursors.Default;
                }
            }
        }

        private void changeQuality_Click(object sender, EventArgs e)
        {
            ChangeQuality();
        }

        private void ExecuteWithoutGet()
        {
            if (MessageBox.Show("Executing a build without get will set some attributes in the TFSBuild.proj file and will revert them back.\r\n" +
                "Are you sure that you want to you want to continue with this operation?", "TFSBuildManager", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
            {
                Cursor = Cursors.WaitCursor;
                try
                {
                    TabBuildType.Resumable = true;
                    try
                    {
                        TabBuildType.Execute(true);
                    }
                    catch (Exception ex)
                    {
                        MessageBox.Show("Error starting build. " + ex.Message, "TFSBuildMnager", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                    finally
                    {
                        TabBuildType.Resumable = false;
                        RefreshView();
                    }
                }
                catch (Exception ex)
                {
                    MessageBox.Show("Error executing build without get. " + ex.Message, "TFSBuildManager",
                        MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                finally
                {
                    Cursor = Cursors.Default;
                }
            }
        }

        private void executeWithoutGet_Click(object sender, EventArgs e)
        {
            ExecuteWithoutGet();
        }

        private void ExploreDropLocation()
        {
            try
            {
                BuildData buildData = (BuildData)TabListView.SelectedItems[0].Tag;
                Process.Start("explorer", "/e," + buildData.DropLocation);
            }
            catch (Exception ex)
            {
                MessageBox.Show("Error exploring build drop location. " + ex.Message, "TFSBuildMnager", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void exploreDropLocation_Click(object sender, EventArgs e)
        {
            ExploreDropLocation();
        }

        private void ViewLogFile()
        {
            try
            {
                BuildData buildData = (BuildData)TabListView.SelectedItems[0].Tag;
                if ((logFileViewer == null) || (logFileViewer == String.Empty))
                {
                    logFileViewer = "notepad";
                }
                string logFile = buildData.DropLocation + @"\BuildLog.txt";
                if (File.Exists(logFile))
                {
                    Process.Start(logFileViewer, logFile);
                }
                else
                {
                    MessageBox.Show(String.Format("The file {0} does not exist.", logFile), "TFSBuildMnager", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show("Error opening log file. " + ex.Message, "TFSBuildMnager", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void viewLogFile_Click(object sender, EventArgs e)
        {
            ViewLogFile();
        }

        private void StartBuildWithProgressForm()
        {
            BuildData buildData = ((BuildData)TabListView.SelectedItems[0].Tag);
            BuildParameters buildParameters = new BuildParameters();
            buildParameters.TeamProject = projectName;
            buildParameters.BuildType = buildData.BuildType;
            buildParameters.BuildDirectory = tabBuildType.BuildDirectoryPath.PropertyValue;
            buildParameters.BuildMachine = tabBuildType.BuildMachine.PropertyValue;
            buildParameters.TeamFoundationServer = sourceControlWrapper.TFS.Uri.AbsoluteUri;
            BuildProgressForm frmBuildProgress = new BuildProgressForm(buildParameters, sourceControlWrapper.TFS.Uri.AbsoluteUri);
            frmBuildProgress.ShowDialog();
        }

        private void ViewDetails()
        {
            this.Cursor = Cursors.WaitCursor;
            try
            {
                BuildData buildData = ((BuildData)TabListView.SelectedItems[0].Tag);
                BuildReportTabControlItem item = new BuildReportTabControlItem(buildStoreWrapper.BS,
                    buildData);
                TabControl.TabControl tabControl = (TabControl.TabControl)this.Parent;
                item.LogFileLinkLabelClicked += new LogFileLinkLabelClickedHandler(item_LogFileLinkLabelClicked);
                tabControl.Items.Add(item);
                tabControl.SelectedItem = item;
            }
            finally
            {
                this.Cursor = Cursors.Default;
            }
        }

        void item_LogFileLinkLabelClicked(LogFileLinkLabelClickedEventArgs e)
        {
            try
            {
                if ((logFileViewer == null) || (logFileViewer == String.Empty))
                {
                    logFileViewer = "notepad";
                }
                if (File.Exists(e.LogFile))
                {
                    Process.Start(logFileViewer, e.LogFile);
                }
                else
                {
                    MessageBox.Show(String.Format("The file {0} does not exist.", e.LogFile), "TFSBuildMnager", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show("Error opening log file. " + ex.Message, "TFSBuildMnager", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }            
        }

        private void viewDetails_Click(object sender, EventArgs e)
        {
            ViewDetails();
        }

        private void GenerateBugsReport()
        {
            GenerateBugsReport(-1);
        }

        //private const string ErrorsTitleLine = "<tr bgcolor=\"red\"><th colspan=\"6\"><font color=\"white\">Build {0} Errors</font></th></tr>";
        //private const string WorkItemTitleLine = "<tr bgcolor=\"#99ccff\"><th colspan=\"4\"><font color=\"black\">Work Item {0} - {1}</font></th></tr>";
        private const string WorkItemLinkLine = "<font color=\"blue\" onclick=\"toggle('{0}');\" style=\"cursor:hand\">Work Item {1} - {2} ({3})</font>";
        private const string WorkItemTableLine = "<table width=\"100%\" border=\"1\" bordercolor=\"black\" cellpadding=\"2\" cellspacing=\"0\" id=\"{0}\" style=\"display:none;\">";
        private const string WorkItemLine = "<tr bgcolor=\"white\"><td><font color=\"black\">&nbsp;{0}</font></td><td><font color=\"black\">&nbsp;{1}</font></td></tr>";
        //private const string ChangesetCommentLine = "<tr bgcolor=\"white\"><td colspan=\"4\"><font color=\"black\">Check In Comment: &nbsp;{0}</font></td></tr>";
        //private const string WorkItemsTitleLine = "<tr bgcolor=\"#99ccff\"><th><font color=\"black\">Id</font></th><th><font color=\"black\">Title</font></th><th><font color=\"black\">State</font></th><th><font color=\"black\">Assigned To</font></th></tr>";
        //private const string WorkItemsLine = "<tr bgcolor=\"white\"><td><font color=\"black\">&nbsp;{0}</font></td><td><font color=\"black\">&nbsp;{1}</font></td><td><font color=\"black\">&nbsp;{2}</font></td><td><font color=\"black\">&nbsp;{3}</font></td></tr>";
 
        private void GenerateBugsReport(int lastReportedChangeSetId)
        {
            this.Cursor = Cursors.WaitCursor;
            try
            {
                BuildData buildData = ((BuildData)TabListView.SelectedItems[0].Tag);
                TFSBuildReport buildReport = new TFSBuildReport(buildStoreWrapper.BS,
                    buildData.BuildUri);
                string reportFileName = Path.Combine(Path.GetTempPath(), "BugsReport.html");
                StreamWriter streamWriter = new StreamWriter(reportFileName);
                try
                {
                    streamWriter.WriteLine("<html><head><title>Build Work Items Report</title></head><body>");
                    streamWriter.WriteLine("<script type=\"text/javascript\">");
                    streamWriter.WriteLine("<!--");
                    streamWriter.WriteLine("function toggle(tableId)");
	                streamWriter.WriteLine("{");
                    streamWriter.WriteLine("var elm = document.getElementById(tableId);");
		            streamWriter.WriteLine("if (elm.style.display == \"none\")");
			        streamWriter.WriteLine("elm.style.display = \"\";");
		            streamWriter.WriteLine("else");
			        streamWriter.WriteLine("elm.style.display =\"none\";");
	                streamWriter.WriteLine("}");
                    streamWriter.WriteLine("//>");
                    streamWriter.WriteLine("</script>");

                    foreach (WorkItemData workItemData in buildReport.WorkItems)
                    {
                        //streamWriter.WriteLine("<table width=\"100%\" border=\"1\" bordercolor=\"black\" cellpadding=\"2\" cellspacing=\"0\">");
                        WorkItemStore workItemStore = (WorkItemStore)sourceControlWrapper.TFS.GetService(typeof(WorkItemStore));
                        WorkItem workItem = workItemStore.GetWorkItem(workItemData.WorkItemId);
                        string tableId = "workItemTbl" + workItem.Id;
                        streamWriter.WriteLine(String.Format(WorkItemLinkLine, tableId, workItem.Id, workItem.Title, workItem.State));
                        streamWriter.WriteLine("<table width=\"100%\" border=\"1\" bordercolor=\"black\" cellpadding=\"2\" cellspacing=\"0\">");
                        streamWriter.WriteLine(String.Format(WorkItemTableLine, tableId)); 
                        streamWriter.WriteLine(String.Format(WorkItemLine, "State", workItem.State));
                        streamWriter.WriteLine(String.Format(WorkItemLine, "Assigned To", workItem.Fields["Assigned To"].Value));
                        streamWriter.WriteLine(String.Format(WorkItemLine, "Changed By ", workItem.ChangedBy));
                        streamWriter.WriteLine("</table>");

                        
                        /*if (!String.IsNullOrEmpty(changeset.Comment))
                        {
                            streamWriter.WriteLine(String.Format(ChangesetCommentLine, changeset.Comment));
                        }
                        WorkItem[] workItems = changeset.WorkItems;
                        if (workItems.Length > 0)
                        {
                            streamWriter.WriteLine(WorkItemsTitleLine);
                            foreach (WorkItem workItem in workItems)
                            {
                                //string text = workItem.State + " " + workItem.Fields["Assigned To"].Value;
                                streamWriter.WriteLine(String.Format(WorkItemsLine, workItem.Id, workItem.Title,
                                    workItem.State, workItem.Fields["Assigned To"].Value));
                            }
                        }*/
                        //streamWriter.WriteLine("</table>");
                        streamWriter.WriteLine("<br>");
                        streamWriter.WriteLine("<br>");
                    }
                    
                    /*foreach (ChangeSetData changeSetData in buildReport.Changesets)
                    {
                        streamWriter.WriteLine("<table width=\"100%\" border=\"1\" bordercolor=\"black\" cellpadding=\"2\" cellspacing=\"0\">");
                        Changeset changeset = sourceControlWrapper.VCS.GetChangeset(changeSetData.ChangeSetId);
                        streamWriter.WriteLine(String.Format(ChangesetTitleLine, changeset.ChangesetId, changeset.Committer));
                        if (!String.IsNullOrEmpty(changeset.Comment))
                        {
                            streamWriter.WriteLine(String.Format(ChangesetCommentLine, changeset.Comment));
                        }
                        WorkItem[] workItems = changeset.WorkItems;
                        if (workItems.Length > 0)
                        {
                            streamWriter.WriteLine(WorkItemsTitleLine);
                            foreach (WorkItem workItem in workItems)
                            {
                                //string text = workItem.State + " " + workItem.Fields["Assigned To"].Value;
                                streamWriter.WriteLine(String.Format(WorkItemsLine, workItem.Id, workItem.Title,
                                    workItem.State, workItem.Fields["Assigned To"].Value));
                            }
                        }
                        streamWriter.WriteLine("</table>");
                        streamWriter.WriteLine("<br>");
                        streamWriter.WriteLine("<br>");
                    }*/
                    streamWriter.WriteLine("</body></html>");
                }
                finally
                {
                    streamWriter.Close();
                    Process.Start(reportFileName);
                }
            }
            finally
            {
                this.Cursor = Cursors.Default;
            }
        }

        private void generateBugsReport_Click(object sender, EventArgs e)
        {
            GenerateBugsReport();
        }

        public void GroupBy(BuildTypeGroupBy groupBy)
        {
            Cursor = Cursors.WaitCursor;
            try
            {
                TabListView.Groups.Clear();
                switch (groupBy)
                {
                    case BuildTypeGroupBy.BuildStatus:
                        {
                            string[] buildStatuses = BuildStoreWrapper.GetBuildStatuses();
                            foreach (string buildStatus in buildStatuses)
                            {
                                ListViewGroup listViewGroup = new ListViewGroup(buildStatus, HorizontalAlignment.Left);
                                listViewGroup.Name = buildStatus.Replace(" ", "");
                                TabListView.Groups.Add(listViewGroup);
                            }
                            foreach (ListViewItem listViewItem in TabListView.Items)
                            {
                                listViewItem.Group = TabListView.Groups[((BuildData)listViewItem.Tag).BuildStatus.Replace(" ", "")];
                            }
                            break;
                        }
                    case BuildTypeGroupBy.BuildQuality:
                        {
                            string[] buildQualities = buildStoreWrapper.GetBuildQualities();
                            foreach (string buildQuality in buildQualities)
                            {
                                ListViewGroup listViewGroup = new ListViewGroup(buildQuality, HorizontalAlignment.Left);
                                listViewGroup.Name = buildQuality.Replace(" ", "");
                                TabListView.Groups.Add(listViewGroup);
                            }
                            foreach (ListViewItem listViewItem in TabListView.Items)
                            {
                                listViewItem.Group = TabListView.Groups[((BuildData)listViewItem.Tag).BuildQuality.Replace(" ", "")];
                            }
                            break;
                        }
                    case BuildTypeGroupBy.CompleteDate:
                        {
                            foreach (ListViewItem listViewItem in TabListView.Items)
                            {
                                BuildData buildData = ((BuildData)listViewItem.Tag);
                                string groupHeader = buildData.FinishTime.ToShortDateString();
                                string groupName = "group" + groupHeader.Replace("/", "");
                                ListViewGroup listViewGroup = TabListView.Groups[groupName];
                                if (listViewGroup == null)
                                {
                                    listViewGroup = new ListViewGroup(groupHeader, HorizontalAlignment.Left);
                                    listViewGroup.Name = groupName;
                                    TabListView.Groups.Add(listViewGroup);
                                }
                                listViewItem.Group = listViewGroup;
                            }
                            break;
                        }
                }
            }
            finally
            {
                Cursor = Cursors.Default;
            }
        }
    }
}
