using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.TeamFoundation.Client;
using Microsoft.TeamFoundation.WorkItemTracking.Client;
using System.Collections.Specialized;

namespace TFSWrappers
{
    public class WorkItemStoreWrapper
    {
        public WorkItemStoreWrapper(ConfigurationType configurationType, string userName, string password, string domain)
        {
            NameValueCollection appSettings = TFSFunctions.GetTFSAppSettingsSection(configurationType);
            string serverName = appSettings[TFSConstants.ServerAppSettingsValueName];
            tfs = TFSFunctions.GetServer(serverName, userName, password, domain);
            wis = (WorkItemStore)tfs.GetService(typeof(WorkItemStore));
        }

        public WorkItemStoreWrapper(ConfigurationType configurationType)
        {
            NameValueCollection appSettings = TFSFunctions.GetTFSAppSettingsSection(configurationType);
            string serverName = appSettings[TFSConstants.ServerAppSettingsValueName];
            tfs = TFSFunctions.GetServer(serverName);
            wis = (WorkItemStore)tfs.GetService(typeof(WorkItemStore));
        }

        public WorkItemStoreWrapper(string serverName)
        {
            tfs = TFSFunctions.GetServer(serverName);
            wis = (WorkItemStore)tfs.GetService(typeof(WorkItemStore));
        }

        public WorkItemStoreWrapper(TeamFoundationServer tfs)
        {
            this.tfs = tfs;
            wis = (WorkItemStore)tfs.GetService(typeof(WorkItemStore));
        }

        private TeamFoundationServer tfs;

        public TeamFoundationServer TFS
        {
            get { return tfs; }
        }

        private WorkItemStore wis;

        public WorkItemStore WIS
        {
            get { return wis; }
        }

        public WorkItemCollection GetWorkItems()
        {
            return wis.Query("Select ID, Title, Attachment From Bug");
        }

        public void AddWorkItem(string projectName, string workItemType, NameValueCollection fields)
        {
            Project wisProject = wis.Projects[projectName];
            WorkItem workItem = new WorkItem(wisProject.WorkItemTypes[workItemType]);
            foreach (string fieldName in fields.Keys)
            {
                workItem.Fields[fieldName].Value = fields[fieldName];
            }
            workItem.Save();
        }        
    }
}
